@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2003 Red Hat, Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,22 +22,37 @@
  */
 #include "bus.h"
 #include <dbus/dbus-internals.h>
+#include <dbus/dbus-watch.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
 #include <errno.h>
+#include "selinux.h"
 
 static BusContext *context;
-static dbus_bool_t got_sighup = FALSE;
+
+static int reload_pipe[2];
+#define RELOAD_READ_END 0
+#define RELOAD_WRITE_END 1
+
 
 static void
 signal_handler (int sig)
 {
+  DBusString str;
+
   switch (sig)
     {
     case SIGHUP:
-      got_sighup = TRUE;
+      _dbus_string_init_const (&str, "foo");
+      if (!_dbus_write (reload_pipe[RELOAD_WRITE_END], &str, 0, 1))
+	{
+	  _dbus_warn ("Unable to write to reload pipe.\n");
+	  exit (1);
+	}
+      break;
+
     case SIGTERM:
       _dbus_loop_quit (bus_context_get_loop (context));
       break;
@@ -98,6 +113,80 @@ check_two_pid_descriptors (const DBusString *pid_fd,
     }
 }
 
+static dbus_bool_t
+handle_reload_watch (DBusWatch    *watch,
+		     unsigned int  flags,
+		     void         *data)
+{
+  DBusError error;
+  DBusString str;
+  _dbus_string_init (&str);
+  if (_dbus_read (reload_pipe[RELOAD_READ_END], &str, 1) != 1)
+    {
+      _dbus_warn ("Couldn't read from reload pipe.\n");
+      exit (1);
+    }
+  _dbus_string_free (&str);
+
+  dbus_error_init (&error);
+  if (! bus_context_reload_config (context, &error))
+    {
+      _dbus_warn ("Unable to reload configuration: %s\n",
+		  error.message);
+      dbus_error_free (&error);
+      exit (1);
+    }
+  return TRUE;
+}
+
+static dbus_bool_t
+reload_watch_callback (DBusWatch    *watch,
+		       unsigned int  condition,
+		       void         *data)
+{
+  return dbus_watch_handle (watch, condition);
+}
+
+static void
+setup_reload_pipe (DBusLoop *loop)
+{
+  DBusError error;
+  DBusWatch *watch;
+
+  dbus_error_init (&error);
+
+  if (!_dbus_full_duplex_pipe (&reload_pipe[0], &reload_pipe[1],
+			       TRUE, &error))
+    {
+      _dbus_warn ("Unable to create reload pipe: %s\n",
+		  error.message);
+      dbus_error_free (&error);
+      exit (1);
+    }
+
+  watch = _dbus_watch_new (reload_pipe[RELOAD_READ_END],
+			   DBUS_WATCH_READABLE, TRUE,
+			   handle_reload_watch, NULL, NULL);
+
+  if (watch == NULL)
+    {
+      _dbus_warn ("Unable to create reload watch: %s\n",
+		  error.message);
+      dbus_error_free (&error);
+      exit (1);
+    }
+
+  if (!_dbus_loop_add_watch (loop, watch, reload_watch_callback,
+			     NULL, NULL))
+    {
+      _dbus_warn ("Unable to add reload watch to main loop: %s\n",
+		  error.message);
+      dbus_error_free (&error);
+      exit (1);
+    }
+
+}
+
 int
 main (int argc, char **argv)
 {
@@ -262,6 +351,7 @@ main (int argc, char **argv)
           print_addr_fd = val;
         }
     }
+  _dbus_string_free (&addr_fd);
 
   print_pid_fd = -1;
   if (print_pid)
@@ -283,7 +373,14 @@ main (int argc, char **argv)
           print_pid_fd = val;
         }
     }
-  
+  _dbus_string_free (&pid_fd);
+
+  if (!bus_selinux_init ())
+    {
+      _dbus_warn ("SELinux initialization failed\n");
+      exit (1);
+    }
+
   dbus_error_init (&error);
   context = bus_context_new (&config_file, force_fork,
                              print_addr_fd, print_pid_fd,
@@ -296,8 +393,10 @@ main (int argc, char **argv)
       dbus_error_free (&error);
       exit (1);
     }
-  
-  /* FIXME we have to handle this properly below _dbus_set_signal_handler (SIGHUP, signal_handler); */
+
+  setup_reload_pipe (bus_context_get_loop (context));
+ 
+  _dbus_set_signal_handler (SIGHUP, signal_handler);
   _dbus_set_signal_handler (SIGTERM, signal_handler);
   
   _dbus_verbose ("We are on D-Bus...\n");
@@ -305,14 +404,7 @@ main (int argc, char **argv)
   
   bus_context_shutdown (context);
   bus_context_unref (context);
+  bus_selinux_shutdown ();
 
-  /* If we exited on TERM we just exit, if we exited on
-   * HUP we restart the daemon.
-   */
-  if (got_sighup)
-    {
-      /* FIXME execv (argv) basically */
-    }
-  
   return 0;
 }
