@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2003 Red Hat, Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include <dbus/dbus-string.h>
 #include <dbus/dbus-sysdeps.h>
 #include <dbus/dbus-internals.h>
+#include "selinux.h"
 
 #ifdef DBUS_BUILD_TESTS
 static void
@@ -69,6 +70,9 @@ main (int argc, char **argv)
       return 1;
     }
 
+  if (!bus_selinux_init ())
+    die ("could not init selinux support");
+
   _dbus_string_init_const (&test_data_dir, dir);
 
 #if 0
@@ -119,6 +123,8 @@ main (int argc, char **argv)
   check_memleaks (argv[0]);
   
   printf ("%s: Success\n", argv[0]);
+
+  bus_selinux_shutdown ();
   
   return 0;
 #else /* DBUS_BUILD_TESTS */
