@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2002, 2003 Red Hat Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,6 +72,10 @@ DBusMessage* dbus_message_new_signal        (const char  *path,
 DBusMessage* dbus_message_new_error         (DBusMessage *reply_to,
                                              const char  *error_name,
                                              const char  *error_message);
+DBusMessage* dbus_message_new_error_printf  (DBusMessage *reply_to,
+                                             const char  *error_name,
+                                             const char  *error_format,
+					     ...);
 
 DBusMessage* dbus_message_copy              (const DBusMessage *message);
 
@@ -150,7 +154,7 @@ dbus_bool_t dbus_message_iter_get_args_valist (DBusMessageIter *iter,
 					       va_list          var_args);
 
 
-void                   dbus_message_iter_init            (DBusMessage      *message,
+dbus_bool_t            dbus_message_iter_init            (DBusMessage      *message,
                                                           DBusMessageIter  *iter);
 dbus_bool_t            dbus_message_iter_has_next        (DBusMessageIter  *iter);
 dbus_bool_t            dbus_message_iter_next            (DBusMessageIter  *iter);
@@ -173,10 +177,10 @@ dbus_bool_t            dbus_message_iter_get_custom      (DBusMessageIter  *iter
                                                           unsigned char   **value,
                                                           int              *len);
 
-void        dbus_message_iter_init_array_iterator (DBusMessageIter   *iter,
+dbus_bool_t dbus_message_iter_init_array_iterator (DBusMessageIter   *iter,
 						   DBusMessageIter   *array_iter,
 						   int               *array_type);
-void        dbus_message_iter_init_dict_iterator  (DBusMessageIter   *iter,
+dbus_bool_t dbus_message_iter_init_dict_iterator  (DBusMessageIter   *iter,
 						   DBusMessageIter   *dict_iter);
 dbus_bool_t dbus_message_iter_get_byte_array      (DBusMessageIter   *iter,
 						   unsigned char    **value,
@@ -230,6 +234,8 @@ dbus_bool_t dbus_message_iter_append_double        (DBusMessageIter      *iter,
 						    double                value);
 dbus_bool_t dbus_message_iter_append_string        (DBusMessageIter      *iter,
 						    const char           *value);
+dbus_bool_t dbus_message_iter_append_object_path   (DBusMessageIter      *iter,
+						    const char           *value);
 dbus_bool_t dbus_message_iter_append_custom        (DBusMessageIter      *iter,
 						    const char           *name,
 						    const unsigned char  *data,
@@ -269,6 +275,9 @@ dbus_bool_t dbus_message_iter_append_byte_array    (DBusMessageIter      *iter,
 dbus_bool_t dbus_message_iter_append_string_array  (DBusMessageIter      *iter,
 						    const char          **value,
 						    int                   len);
+dbus_bool_t dbus_message_iter_append_object_path_array (DBusMessageIter      *iter,
+							const char          **value,
+							int                   len);
 
 
 
