@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2003  Red Hat Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -190,11 +190,14 @@ ensure_sorted (DBusObjectSubtree *subtree)
 static DBusObjectSubtree*
 find_subtree_recurse (DBusObjectSubtree  *subtree,
                       const char        **path,
-                      dbus_bool_t         return_deepest_match,
                       dbus_bool_t         create_if_not_found,
-                      int                *index_in_parent)
+                      int                *index_in_parent,
+                      dbus_bool_t        *exact_match)
 {
   int i;
+  dbus_bool_t return_deepest_match;
+
+  return_deepest_match = exact_match != NULL;
 
   _dbus_assert (!(return_deepest_match && create_if_not_found));
 
@@ -204,6 +207,8 @@ find_subtree_recurse (DBusObjectSubtree  *subtree,
       _dbus_verbose ("  path exhausted, returning %s\n",
                      subtree->name);
 #endif
+      if (exact_match != NULL)
+	*exact_match = TRUE;
       return subtree;
     }
 
@@ -246,8 +251,8 @@ find_subtree_recurse (DBusObjectSubtree  *subtree,
               DBusObjectSubtree *next;
 
               next = find_subtree_recurse (subtree->subtrees[i],
-                                           &path[1], return_deepest_match,
-                                           create_if_not_found, index_in_parent);
+                                           &path[1], create_if_not_found, 
+                                           index_in_parent, exact_match);
               if (next == NULL &&
                   subtree->invoke_as_fallback)
                 {
@@ -255,6 +260,8 @@ find_subtree_recurse (DBusObjectSubtree  *subtree,
                   _dbus_verbose ("  no deeper match found, returning %s\n",
                                  subtree->name);
 #endif
+		  if (exact_match != NULL)
+		    *exact_match = FALSE;
                   return subtree;
                 }
               else
@@ -262,8 +269,8 @@ find_subtree_recurse (DBusObjectSubtree  *subtree,
             }
           else
             return find_subtree_recurse (subtree->subtrees[i],
-                                         &path[1], return_deepest_match,
-                                         create_if_not_found, index_in_parent);
+                                         &path[1], create_if_not_found, 
+                                         index_in_parent, exact_match);
         }
       else if (v < 0)
         {
@@ -304,7 +311,7 @@ find_subtree_recurse (DBusObjectSubtree  *subtree,
           child->unregister_function = NULL;
           child->message_function = NULL;
           _dbus_object_subtree_unref (child);
-          return FALSE;
+          return NULL;
         }
 
       new_subtrees[subtree->n_subtrees] = child;
@@ -317,11 +324,15 @@ find_subtree_recurse (DBusObjectSubtree  *subtree,
       child->parent = subtree;
 
       return find_subtree_recurse (child,
-                                   &path[1], return_deepest_match,
-                                   create_if_not_found, index_in_parent);
+                                   &path[1], create_if_not_found, 
+                                   index_in_parent, exact_match);
     }
   else
-    return (return_deepest_match && subtree->invoke_as_fallback) ? subtree : NULL;
+    {
+      if (exact_match != NULL)
+	*exact_match = FALSE;
+      return (return_deepest_match && subtree->invoke_as_fallback) ? subtree : NULL;
+    }
 }
 
 static DBusObjectSubtree*
@@ -335,7 +346,7 @@ find_subtree (DBusObjectTree *tree,
   _dbus_verbose ("Looking for exact registered subtree\n");
 #endif
   
-  subtree = find_subtree_recurse (tree->root, path, FALSE, FALSE, index_in_parent);
+  subtree = find_subtree_recurse (tree->root, path, FALSE, index_in_parent, NULL);
 
   if (subtree && subtree->message_function == NULL)
     return NULL;
@@ -343,14 +354,26 @@ find_subtree (DBusObjectTree *tree,
     return subtree;
 }
 
+static DBusObjectSubtree*
+lookup_subtree (DBusObjectTree *tree,
+                const char    **path)
+{
+#if VERBOSE_FIND
+  _dbus_verbose ("Looking for subtree\n");
+#endif
+  return find_subtree_recurse (tree->root, path, FALSE, NULL, NULL);
+}
+
 static DBusObjectSubtree*
 find_handler (DBusObjectTree *tree,
-              const char    **path)
+              const char    **path,
+              dbus_bool_t    *exact_match)
 {
 #if VERBOSE_FIND
   _dbus_verbose ("Looking for deepest handler\n");
 #endif
-  return find_subtree_recurse (tree->root, path, TRUE, FALSE, NULL);
+  _dbus_assert (exact_match != NULL);
+  return find_subtree_recurse (tree->root, path, FALSE, NULL, exact_match);
 }
 
 static DBusObjectSubtree*
@@ -360,7 +383,7 @@ ensure_subtree (DBusObjectTree *tree,
 #if VERBOSE_FIND
   _dbus_verbose ("Ensuring subtree\n");
 #endif
-  return find_subtree_recurse (tree->root, path, FALSE, TRUE, NULL);
+  return find_subtree_recurse (tree->root, path, TRUE, NULL, NULL);
 }
 
 /**
@@ -513,13 +536,12 @@ free_subtree_recurse (DBusConnection    *connection,
 
   /* Call application code */
   if (subtree->unregister_function)
-    {
-      (* subtree->unregister_function) (connection,
-                                        subtree->user_data);
-      subtree->message_function = NULL;
-      subtree->unregister_function = NULL;
-      subtree->user_data = NULL;
-    }
+    (* subtree->unregister_function) (connection,
+				      subtree->user_data);
+
+  subtree->message_function = NULL;
+  subtree->unregister_function = NULL;
+  subtree->user_data = NULL;
 
   /* Now free ourselves */
   _dbus_object_subtree_unref (subtree);
@@ -553,12 +575,10 @@ _dbus_object_tree_list_registered_unlocked (DBusObjectTree *tree,
 
   *child_entries = NULL;
   
-  subtree = find_subtree (tree, parent_path, NULL);
+  subtree = lookup_subtree (tree, parent_path);
   if (subtree == NULL)
     {
       retval = dbus_new0 (char *, 1);
-      if (retval == NULL)
-        goto out;
     }
   else
     {
@@ -653,6 +673,7 @@ _dbus_object_tree_dispatch_and_unlock (DBusObjectTree          *tree,
                                        DBusMessage             *message)
 {
   char **path;
+  dbus_bool_t exact_match;
   DBusList *list;
   DBusList *link;
   DBusHandlerResult result;
@@ -687,7 +708,7 @@ _dbus_object_tree_dispatch_and_unlock (DBusObjectTree          *tree,
     }
   
   /* Find the deepest path that covers the path in the message */
-  subtree = find_handler (tree, (const char**) path);
+  subtree = find_handler (tree, (const char**) path, &exact_match);
   
   /* Build a list of all paths that cover the path in the message */
 
@@ -695,7 +716,7 @@ _dbus_object_tree_dispatch_and_unlock (DBusObjectTree          *tree,
 
   while (subtree != NULL)
     {
-      if (subtree->message_function != NULL)
+      if (subtree->message_function != NULL && (exact_match || subtree->invoke_as_fallback))
         {
           _dbus_object_subtree_ref (subtree);
 
@@ -708,6 +729,7 @@ _dbus_object_tree_dispatch_and_unlock (DBusObjectTree          *tree,
             }
         }
 
+      exact_match = FALSE;
       subtree = subtree->parent;
     }
 
@@ -850,7 +872,8 @@ _dbus_object_subtree_new (const char                  *name,
   subtree->subtrees = NULL;
   subtree->n_subtrees = 0;
   subtree->subtrees_sorted = TRUE;
-  
+  subtree->invoke_as_fallback = FALSE;
+
   return subtree;
 
  oom:
@@ -891,7 +914,7 @@ _dbus_object_subtree_unref (DBusObjectSubtree *subtree)
  * the given parent_path. The returned array should be freed with
  * dbus_free_string_array().
  *
- * @param connection the connection
+ * @param tree the object tree
  * @param parent_path the path to list the child handlers of
  * @param child_entries returns #NULL-terminated array of children
  * @returns #FALSE if no memory to allocate the child entries
@@ -955,9 +978,17 @@ flatten_path (const char **path)
   return NULL;
 }
 
+
+typedef enum 
+{
+  STR_EQUAL,
+  STR_PREFIX,
+  STR_DIFFERENT
+} StrComparison;
+
 /* Returns TRUE if container is a parent of child
  */
-static dbus_bool_t
+static StrComparison
 path_contains (const char **container,
                const char **child)
 {
@@ -969,10 +1000,10 @@ path_contains (const char **container,
       int v;
 
       if (container[i] == NULL)
-        return TRUE; /* container ran out, child continues;
-                      * thus the container is a parent of the
-                      * child.
-                      */
+        return STR_PREFIX; /* container ran out, child continues;
+			* thus the container is a parent of the
+			* child.
+			*/
 
       _dbus_assert (container[i] != NULL);
       _dbus_assert (child[i] != NULL);
@@ -980,9 +1011,9 @@ path_contains (const char **container,
       v = strcmp (container[i], child[i]);
 
       if (v != 0)
-        return FALSE; /* they overlap until here and then are different,
-                       * not overlapping
-                       */
+        return STR_DIFFERENT; /* they overlap until here and then are different,
+			   * not overlapping
+			   */
 
       ++i;
     }
@@ -991,11 +1022,12 @@ path_contains (const char **container,
    * otherwise, the child is a parent of the container.
    */
   if (container[i] == NULL)
-    return TRUE; /* equal is counted as containing */
+    return STR_EQUAL;
   else
-    return FALSE;
+    return STR_DIFFERENT;
 }
 
+#if 0
 static void
 spew_subtree_recurse (DBusObjectSubtree *subtree,
                       int                indent)
@@ -1026,6 +1058,7 @@ spew_tree (DBusObjectTree *tree)
 {
   spew_subtree_recurse (tree->root, 0);
 }
+#endif
 
 /**
  * Callback data used in tests
@@ -1033,9 +1066,9 @@ spew_tree (DBusObjectTree *tree)
 typedef struct
 {
   const char **path; /**< Path */
+  dbus_bool_t handler_fallback; /**< true if the handler may be called as fallback */
   dbus_bool_t message_handled; /**< Gets set to true if message handler called */
   dbus_bool_t handler_unregistered; /**< gets set to true if handler is unregistered */
-
 } TreeTestData;
 
 
@@ -1063,6 +1096,7 @@ test_message_function (DBusConnection  *connection,
 static dbus_bool_t
 do_register (DBusObjectTree *tree,
              const char    **path,
+             dbus_bool_t     fallback,
              int             i,
              TreeTestData   *tree_test_data)
 {
@@ -1071,9 +1105,10 @@ do_register (DBusObjectTree *tree,
 
   tree_test_data[i].message_handled = FALSE;
   tree_test_data[i].handler_unregistered = FALSE;
+  tree_test_data[i].handler_fallback = fallback;
   tree_test_data[i].path = path;
 
-  if (!_dbus_object_tree_register (tree, TRUE, path,
+  if (!_dbus_object_tree_register (tree, fallback, path,
                                    &vtable,
                                    &tree_test_data[i]))
     return FALSE;
@@ -1124,11 +1159,21 @@ do_test_dispatch (DBusObjectTree *tree,
   while (j < n_test_data)
     {
       if (tree_test_data[j].message_handled)
-        _dbus_assert (path_contains (tree_test_data[j].path,
-                                     path));
+	{
+	  if (tree_test_data[j].handler_fallback)
+	    _dbus_assert (path_contains (tree_test_data[j].path,
+					 path) != STR_DIFFERENT);
+	  else
+	    _dbus_assert (path_contains (tree_test_data[j].path, path) == STR_EQUAL);
+	}
       else
-        _dbus_assert (!path_contains (tree_test_data[j].path,
-                                      path));
+	{
+	  if (tree_test_data[j].handler_fallback)
+	    _dbus_assert (path_contains (tree_test_data[j].path,
+					 path) == STR_DIFFERENT);
+	  else
+	    _dbus_assert (path_contains (tree_test_data[j].path, path) != STR_EQUAL);
+	}
 
       ++j;
     }
@@ -1143,6 +1188,15 @@ do_test_dispatch (DBusObjectTree *tree,
   return FALSE;
 }
 
+static size_t
+string_array_length (char **array)
+{
+  size_t i;
+  for (i = 0; array[i]; i++) ;
+  return i;
+}
+
+
 static dbus_bool_t
 object_tree_test_iteration (void *data)
 {
@@ -1157,6 +1211,7 @@ object_tree_test_iteration (void *data)
   DBusObjectTree *tree;
   TreeTestData tree_test_data[8];
   int i;
+  dbus_bool_t exact_match;
 
   tree = NULL;
 
@@ -1164,7 +1219,7 @@ object_tree_test_iteration (void *data)
   if (tree == NULL)
     goto out;
 
-  if (!do_register (tree, path1, 0, tree_test_data))
+  if (!do_register (tree, path1, TRUE, 0, tree_test_data))
     goto out;
 
   _dbus_assert (find_subtree (tree, path1, NULL));
@@ -1176,16 +1231,16 @@ object_tree_test_iteration (void *data)
   _dbus_assert (!find_subtree (tree, path7, NULL));
   _dbus_assert (!find_subtree (tree, path8, NULL));
 
-  _dbus_assert (find_handler (tree, path1));
-  _dbus_assert (find_handler (tree, path2));
-  _dbus_assert (find_handler (tree, path3));
-  _dbus_assert (find_handler (tree, path4));
-  _dbus_assert (find_handler (tree, path5) == tree->root);
-  _dbus_assert (find_handler (tree, path6) == tree->root);
-  _dbus_assert (find_handler (tree, path7) == tree->root);
-  _dbus_assert (find_handler (tree, path8) == tree->root);
+  _dbus_assert (find_handler (tree, path1, &exact_match) &&  exact_match);
+  _dbus_assert (find_handler (tree, path2, &exact_match) && !exact_match);
+  _dbus_assert (find_handler (tree, path3, &exact_match) && !exact_match);
+  _dbus_assert (find_handler (tree, path4, &exact_match) && !exact_match);
+  _dbus_assert (find_handler (tree, path5, &exact_match) == tree->root && !exact_match);
+  _dbus_assert (find_handler (tree, path6, &exact_match) == tree->root && !exact_match);
+  _dbus_assert (find_handler (tree, path7, &exact_match) == tree->root && !exact_match);
+  _dbus_assert (find_handler (tree, path8, &exact_match) == tree->root && !exact_match);
 
-  if (!do_register (tree, path2, 1, tree_test_data))
+  if (!do_register (tree, path2, TRUE, 1, tree_test_data))
     goto out;
 
   _dbus_assert (find_subtree (tree, path1, NULL));
@@ -1197,7 +1252,7 @@ object_tree_test_iteration (void *data)
   _dbus_assert (!find_subtree (tree, path7, NULL));
   _dbus_assert (!find_subtree (tree, path8, NULL));
 
-  if (!do_register (tree, path3, 2, tree_test_data))
+  if (!do_register (tree, path3, TRUE, 2, tree_test_data))
     goto out;
 
   _dbus_assert (find_subtree (tree, path1, NULL));
@@ -1209,7 +1264,7 @@ object_tree_test_iteration (void *data)
   _dbus_assert (!find_subtree (tree, path7, NULL));
   _dbus_assert (!find_subtree (tree, path8, NULL));
   
-  if (!do_register (tree, path4, 3, tree_test_data))
+  if (!do_register (tree, path4, TRUE, 3, tree_test_data))
     goto out;
 
   _dbus_assert (find_subtree (tree, path1, NULL));
@@ -1221,7 +1276,7 @@ object_tree_test_iteration (void *data)
   _dbus_assert (!find_subtree (tree, path7, NULL));
   _dbus_assert (!find_subtree (tree, path8, NULL));
   
-  if (!do_register (tree, path5, 4, tree_test_data))
+  if (!do_register (tree, path5, TRUE, 4, tree_test_data))
     goto out;
 
   _dbus_assert (find_subtree (tree, path1, NULL));
@@ -1233,16 +1288,16 @@ object_tree_test_iteration (void *data)
   _dbus_assert (!find_subtree (tree, path7, NULL));
   _dbus_assert (!find_subtree (tree, path8, NULL));
   
-  _dbus_assert (find_handler (tree, path1) != tree->root);
-  _dbus_assert (find_handler (tree, path2) != tree->root);
-  _dbus_assert (find_handler (tree, path3) != tree->root);
-  _dbus_assert (find_handler (tree, path4) != tree->root);
-  _dbus_assert (find_handler (tree, path5) != tree->root);
-  _dbus_assert (find_handler (tree, path6) != tree->root);
-  _dbus_assert (find_handler (tree, path7) != tree->root);
-  _dbus_assert (find_handler (tree, path8) == tree->root);
+  _dbus_assert (find_handler (tree, path1, &exact_match) != tree->root &&  exact_match);
+  _dbus_assert (find_handler (tree, path2, &exact_match) != tree->root &&  exact_match);
+  _dbus_assert (find_handler (tree, path3, &exact_match) != tree->root &&  exact_match);
+  _dbus_assert (find_handler (tree, path4, &exact_match) != tree->root &&  exact_match);
+  _dbus_assert (find_handler (tree, path5, &exact_match) != tree->root &&  exact_match);
+  _dbus_assert (find_handler (tree, path6, &exact_match) != tree->root && !exact_match);
+  _dbus_assert (find_handler (tree, path7, &exact_match) != tree->root && !exact_match);
+  _dbus_assert (find_handler (tree, path8, &exact_match) == tree->root && !exact_match);
 
-  if (!do_register (tree, path6, 5, tree_test_data))
+  if (!do_register (tree, path6, TRUE, 5, tree_test_data))
     goto out;
 
   _dbus_assert (find_subtree (tree, path1, NULL));
@@ -1254,7 +1309,7 @@ object_tree_test_iteration (void *data)
   _dbus_assert (!find_subtree (tree, path7, NULL));
   _dbus_assert (!find_subtree (tree, path8, NULL));
 
-  if (!do_register (tree, path7, 6, tree_test_data))
+  if (!do_register (tree, path7, TRUE, 6, tree_test_data))
     goto out;
 
   _dbus_assert (find_subtree (tree, path1, NULL));
@@ -1266,7 +1321,7 @@ object_tree_test_iteration (void *data)
   _dbus_assert (find_subtree (tree, path7, NULL));
   _dbus_assert (!find_subtree (tree, path8, NULL));
 
-  if (!do_register (tree, path8, 7, tree_test_data))
+  if (!do_register (tree, path8, TRUE, 7, tree_test_data))
     goto out;
 
   _dbus_assert (find_subtree (tree, path1, NULL));
@@ -1278,15 +1333,55 @@ object_tree_test_iteration (void *data)
   _dbus_assert (find_subtree (tree, path7, NULL));
   _dbus_assert (find_subtree (tree, path8, NULL));
   
-  _dbus_assert (find_handler (tree, path1) != tree->root);
-  _dbus_assert (find_handler (tree, path2) != tree->root);
-  _dbus_assert (find_handler (tree, path3) != tree->root);
-  _dbus_assert (find_handler (tree, path4) != tree->root);
-  _dbus_assert (find_handler (tree, path5) != tree->root);
-  _dbus_assert (find_handler (tree, path6) != tree->root);
-  _dbus_assert (find_handler (tree, path7) != tree->root);
-  _dbus_assert (find_handler (tree, path8) != tree->root);
+  _dbus_assert (find_handler (tree, path1, &exact_match) != tree->root && exact_match);
+  _dbus_assert (find_handler (tree, path2, &exact_match) != tree->root && exact_match);
+  _dbus_assert (find_handler (tree, path3, &exact_match) != tree->root && exact_match);
+  _dbus_assert (find_handler (tree, path4, &exact_match) != tree->root && exact_match);
+  _dbus_assert (find_handler (tree, path5, &exact_match) != tree->root && exact_match);
+  _dbus_assert (find_handler (tree, path6, &exact_match) != tree->root && exact_match);
+  _dbus_assert (find_handler (tree, path7, &exact_match) != tree->root && exact_match);
+  _dbus_assert (find_handler (tree, path8, &exact_match) != tree->root && exact_match);
   
+  /* test the list_registered function */
+
+  {
+    const char *root[] = { NULL };
+    char **child_entries;
+    int nb;
+
+    _dbus_object_tree_list_registered_unlocked (tree, path1, &child_entries);
+    if (child_entries != NULL)
+      {
+	nb = string_array_length (child_entries);
+	_dbus_assert (nb == 1);
+	dbus_free_string_array (child_entries);
+      }
+
+    _dbus_object_tree_list_registered_unlocked (tree, path2, &child_entries);
+    if (child_entries != NULL)
+      {
+	nb = string_array_length (child_entries);
+	_dbus_assert (nb == 2);
+	dbus_free_string_array (child_entries);
+      }
+
+    _dbus_object_tree_list_registered_unlocked (tree, path8, &child_entries);
+    if (child_entries != NULL)
+      {
+	nb = string_array_length (child_entries);
+	_dbus_assert (nb == 0);
+	dbus_free_string_array (child_entries);
+      }
+
+    _dbus_object_tree_list_registered_unlocked (tree, root, &child_entries);
+    if (child_entries != NULL)
+      {
+	nb = string_array_length (child_entries);
+	_dbus_assert (nb == 3);
+	dbus_free_string_array (child_entries);
+      }
+  }
+
   /* Check that destroying tree calls unregister funcs */
   _dbus_object_tree_unref (tree);
 
@@ -1303,21 +1398,21 @@ object_tree_test_iteration (void *data)
   if (tree == NULL)
     goto out;
 
-  if (!do_register (tree, path1, 0, tree_test_data))
+  if (!do_register (tree, path1, TRUE, 0, tree_test_data))
     goto out;
-  if (!do_register (tree, path2, 1, tree_test_data))
+  if (!do_register (tree, path2, TRUE, 1, tree_test_data))
     goto out;
-  if (!do_register (tree, path3, 2, tree_test_data))
+  if (!do_register (tree, path3, TRUE, 2, tree_test_data))
     goto out;
-  if (!do_register (tree, path4, 3, tree_test_data))
+  if (!do_register (tree, path4, TRUE, 3, tree_test_data))
     goto out;
-  if (!do_register (tree, path5, 4, tree_test_data))
+  if (!do_register (tree, path5, TRUE, 4, tree_test_data))
     goto out;
-  if (!do_register (tree, path6, 5, tree_test_data))
+  if (!do_register (tree, path6, TRUE, 5, tree_test_data))
     goto out;
-  if (!do_register (tree, path7, 6, tree_test_data))
+  if (!do_register (tree, path7, TRUE, 6, tree_test_data))
     goto out;
-  if (!do_register (tree, path8, 7, tree_test_data))
+  if (!do_register (tree, path8, TRUE, 7, tree_test_data))
     goto out;
   
   _dbus_object_tree_unregister_and_unlock (tree, path1);
@@ -1418,21 +1513,21 @@ object_tree_test_iteration (void *data)
 
   /* Register it all again, and test dispatch */
 
-  if (!do_register (tree, path1, 0, tree_test_data))
+  if (!do_register (tree, path1, FALSE, 0, tree_test_data))
     goto out;
-  if (!do_register (tree, path2, 1, tree_test_data))
+  if (!do_register (tree, path2, TRUE, 1, tree_test_data))
     goto out;
-  if (!do_register (tree, path3, 2, tree_test_data))
+  if (!do_register (tree, path3, TRUE, 2, tree_test_data))
     goto out;
-  if (!do_register (tree, path4, 3, tree_test_data))
+  if (!do_register (tree, path4, TRUE, 3, tree_test_data))
     goto out;
-  if (!do_register (tree, path5, 4, tree_test_data))
+  if (!do_register (tree, path5, TRUE, 4, tree_test_data))
     goto out;
-  if (!do_register (tree, path6, 5, tree_test_data))
+  if (!do_register (tree, path6, FALSE, 5, tree_test_data))
     goto out;
-  if (!do_register (tree, path7, 6, tree_test_data))
+  if (!do_register (tree, path7, TRUE, 6, tree_test_data))
     goto out;
-  if (!do_register (tree, path8, 7, tree_test_data))
+  if (!do_register (tree, path8, TRUE, 7, tree_test_data))
     goto out;
 
 #if 0
