@@ -1,9 +1,9 @@
 /* -*- mode: C; c-file-style: "gnu" -*- */
 /* dbus-server.c DBusServer object
  *
- * Copyright (C) 2002, 2003 Red Hat Inc.
+ * Copyright (C) 2002, 2003, 2004 Red Hat Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "dbus-server-debug-pipe.h"
 #endif
 #include "dbus-address.h"
+#include "dbus-protocol.h"
 
 /**
  * @defgroup DBusServer DBusServer
@@ -455,7 +456,7 @@ dbus_server_listen (const char     *address,
 DBusServer *
 dbus_server_ref (DBusServer *server)
 {
-  _dbus_return_if_fail (server != NULL);
+  _dbus_return_val_if_fail (server != NULL, NULL);
   
   server->refcount += 1;
 
@@ -788,3 +789,42 @@ dbus_server_get_data (DBusServer   *server,
 
 /** @} */
 
+#ifdef DBUS_BUILD_TESTS
+#include "dbus-test.h"
+
+dbus_bool_t
+_dbus_server_test (void)
+{
+  const char *valid_addresses[] = {
+    "tcp:port=1234",
+    "unix:path=./boogie",
+    "tcp:host=localhost,port=1234",
+    "tcp:host=localhost,port=1234;tcp:port=5678",
+    "tcp:port=1234;unix:path=./boogie",
+  };
+
+  DBusServer *server;
+  int i;
+  
+  for (i = 0; i < _DBUS_N_ELEMENTS (valid_addresses); i++)
+    {
+      server = dbus_server_listen (valid_addresses[i], NULL);
+      if (server == NULL)
+	_dbus_assert_not_reached ("Failed to listen for valid address.");
+
+      dbus_server_unref (server);
+
+      /* Try disconnecting before unreffing */
+      server = dbus_server_listen (valid_addresses[i], NULL);
+      if (server == NULL)
+	_dbus_assert_not_reached ("Failed to listen for valid address.");
+
+      dbus_server_disconnect (server);
+
+      dbus_server_unref (server);
+    }
+
+  return TRUE;
+}
+
+#endif /* DBUS_BUILD_TESTS */
