@@ -4,7 +4,7 @@
  * Copyright (C) 2002, 2003  Red Hat, Inc.
  * Copyright (C) 2003 CodeFactory AB
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "dbus-internals.h"
 #include "dbus-sysdeps.h"
 #include "dbus-threads.h"
+#include "dbus-protocol.h"
 #include "dbus-test.h"
 #include <sys/types.h>
 #include <stdlib.h>
@@ -687,9 +688,6 @@ _dbus_listen_tcp_socket (const char     *host,
       return -1;
     }
 
-  if (host == NULL)
-    host = "localhost";
-  
   he = gethostbyname (host);
   if (he == NULL) 
     {
@@ -1551,7 +1549,7 @@ fill_user_info (DBusUserInfo       *info,
     if (info->group_ids == NULL)
       {
         dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
-        goto out;
+        goto failed;
       }
 
     info->n_group_ids = 1;
@@ -1915,8 +1913,6 @@ _dbus_atomic_dec (DBusAtomic *atomic)
 /**
  * Wrapper for poll().
  *
- * @todo need a fallback implementation using select()
- *
  * @param fds the file descriptors to poll
  * @param n_fds number of descriptors in the array
  * @param timeout_milliseconds timeout or -1 for infinite
